public class Graf {
    private final Node[] nodes;
    private final MaxHeap<Node> heap = new MaxHeap<>();

    Graf(int n) {
        this.nodes = new Node[n];
        for (int i = 0; i < n; i++) {
            Node node = new Node(i);
            nodes[i] = node;
            heap.insert(node);
        }
    }

    void povezi(int u, int v) {
        nodes[u].degree++;
        nodes[v].degree++;
        nodes[u].neighbors.add(nodes[v]);
        nodes[v].neighbors.add(nodes[u]);
//        heap.heapify(u);
//        heap.heapify(v);
        heap.rebuild();
    }

    void pozresni() {
        ArrList<Integer> used = new ArrList<>();

        while (!heap.isEmpty()) {
            Node node = heap.pop();
            if (node.degree == 0) {
                break;
            }

            LinkedList<Node>.Node neighbor = node.neighbors.root;
            do {
                neighbor.value.degree--;
                heap.heapify(neighbor.value.id);
                neighbor = neighbor.next;
            } while (neighbor != null);

            used.add(node.id);
        }

        print(used);
    }

    private void print(ArrList<Integer> arr) {
        int idxStart = arr.size() > 100 ? arr.size() - 100 : 0;
        for (int i = idxStart; i < arr.size(); i++) {
            System.out.print(arr.get(i));
            if (i < arr.size() - 1) System.out.print(" ");
        }
        System.out.println();
    }
}

class Node implements Comparable<Node> {
    int id;
    int degree = 0;
    LinkedList<Node> neighbors = new LinkedList<>();

    Node(int id) {
        this.id = id;
    }

    @Override
    public int compareTo(Node other) {
        int deg = Integer.compare(this.degree, other.degree);
        if (deg == 0) {
            return Integer.compare(other.id, this.id);
        }
        return deg;
    }

    @Override
    public String toString() {
        return "Node [id=" + id + ", degree=" + degree + "]";
    }
}

class MaxHeap<T extends Comparable<T>> {
    final private ArrList<T> arr;

    public MaxHeap() {
        arr = new ArrList<>();
    }

    public MaxHeap(MaxHeap<T> heap) {
        this.arr = new ArrList<>(heap.arr);
    }

    private int parent(int i) {
        return (i - 1) / 2;
    }

    private int leftChild(int i) {
        return 2 * i + 1;
    }

    private int rightChild(int i) {
        return 2 * i + 2;
    }

    private void swap(int i, int j) {
        T tmp = arr.get(i);
        arr.set(i, arr.get(j));
        arr.set(j, tmp);
    }

    public void heapify(int i) {
        int largest = i;
        int left = leftChild(i);
        int right = rightChild(i);

        if (left < arr.size() && arr.get(left).compareTo(arr.get(largest)) > 0) {
            largest = left;
        }

        if (right < arr.size() && arr.get(right).compareTo(arr.get(largest)) > 0) {
            largest = right;
        }

        if (largest != i) {
            swap(i, largest);
            heapify(largest);
        }
    }

    public void insert(T x) {
        arr.add(x);
        int currentIndex = arr.size() - 1;
        while (currentIndex > 0 && arr.get(currentIndex).compareTo(arr.get(parent(currentIndex))) > 0) {
            swap(currentIndex, parent(currentIndex));
            currentIndex = parent(currentIndex);
        }
    }

    public boolean isEmpty() {
        return arr.isEmpty();
    }

    public void rebuild() {
        for (int i = parent(arr.size() - 1); i >= 0; i--) {
            heapify(i);
        }
    }

    public T pop() {
        if (arr.isEmpty()) {
            throw new RuntimeException("Heap is empty");
        }

        T max = arr.get(0);
        T lastElement = arr.removeLast();

        if (!arr.isEmpty()) {
            arr.set(0, lastElement);

            int currentIndex = 0;
            while (true) {
                int left = leftChild(currentIndex);
                int right = rightChild(currentIndex);

                int largest = currentIndex;

                if (left < arr.size() && arr.get(left).compareTo(arr.get(largest)) > 0) {
                    largest = left;
                }

                if (right < arr.size() && arr.get(right).compareTo(arr.get(largest)) > 0) {
                    largest = right;
                }

                if (largest == currentIndex) {
                    break;
                }

                swap(currentIndex, largest);

                currentIndex = largest;
            }
        }

        return max;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < arr.size(); i++) {
            sb.append(arr.get(i)).append(" ");
        }
        return sb.toString();
    }
}

class LinkedList<T> {
    class Node {
        T value;
        Node next;

        Node(T value) {
            this.value = value;
        }
    }

    Node root;

    public void add(T value) {
        Node node = new Node(value);
        node.next = root;
        root = node;
    }
}

class ArrList<T> {
    private T[] array;
    private int size;

    @SuppressWarnings("unchecked")
    public ArrList() {
        array = (T[]) new Object[64];
        size = 0;
    }

    @SuppressWarnings("unchecked")
    public ArrList(ArrList<T> list) {
        this.size = list.size;
        this.array = (T[]) new Object[list.array.length];
        System.arraycopy(list.array, 0, this.array, 0, list.size);
    }

    private boolean isFull() {
        return size == array.length;
    }

    private void resize() {
        @SuppressWarnings("unchecked")
        T[] tmp = (T[]) new Object[array.length * 2];
        System.arraycopy(array, 0, tmp, 0, size);
        array = tmp;
    }

    public T get(int i) {
        return array[i];
    }

    public void add(T t) {
        if (isFull()) {
            resize();
        }
        array[size++] = t;
    }

    public void set(int i, T t) {
        array[i] = t;
    }

    public T removeLast() {
        T tmp = array[--size];
        array[size] = null;
        return tmp;
    }

    public int size() {
        return size;
    }

    public boolean isEmpty() {
        return size == 0;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < size; i++) {
            sb.append(array[i]).append(" ");
        }
        return sb.toString();
    }
}
